const getDescriptor = (o, p) => Object.getOwnPropertyDescriptor(o, p);
const nodeProto = Node.prototype;
const characterDataProto = CharacterData.prototype;
const nodeInsertBefore = nodeProto.insertBefore;
const characterDataSetData = getDescriptor(characterDataProto, "data").set;
const nodeRemoveChild = nodeProto.removeChild;
class VSimpleNode {
    constructor(text) {
        this.text = text;
    }
    mountNode(node, parent, afterNode) {
        this.parentEl = parent;
        nodeInsertBefore.call(parent, node, afterNode);
        this.el = node;
    }
    moveBefore(other, afterNode) {
        const target = other ? other.el : afterNode;
        nodeInsertBefore.call(this.parentEl, this.el, target);
    }
    beforeRemove() { }
    remove() {
        nodeRemoveChild.call(this.parentEl, this.el);
    }
    firstNode() {
        return this.el;
    }
    toString() {
        return this.text;
    }
}
class VText extends VSimpleNode {
    mount(parent, afterNode) {
        this.mountNode(document.createTextNode(toText(this.text)), parent, afterNode);
    }
    patch(other) {
        const text2 = other.text;
        if (this.text !== text2) {
            characterDataSetData.call(this.el, toText(text2));
            this.text = text2;
        }
    }
}
class VComment extends VSimpleNode {
    mount(parent, afterNode) {
        this.mountNode(document.createComment(toText(this.text)), parent, afterNode);
    }
    patch() { }
}
export function text(str) {
    return new VText(str);
}
export function comment(str) {
    return new VComment(str);
}
export function toText(value) {
    switch (typeof value) {
        case "string":
            return value;
        case "number":
            return String(value);
        case "boolean":
            return value ? "true" : "false";
        default:
            return value || "";
    }
}
