// -----------------------------------------------------------------------------
// Toggler node
// -----------------------------------------------------------------------------
class VToggler {
    constructor(key, child) {
        this.key = key;
        this.child = child;
    }
    mount(parent, afterNode) {
        this.parentEl = parent;
        this.child.mount(parent, afterNode);
    }
    moveBefore(other, afterNode) {
        this.child.moveBefore(other ? other.child : null, afterNode);
    }
    patch(other, withBeforeRemove) {
        if (this === other) {
            return;
        }
        let child1 = this.child;
        let child2 = other.child;
        if (this.key === other.key) {
            child1.patch(child2, withBeforeRemove);
        }
        else {
            child2.mount(this.parentEl, child1.firstNode());
            if (withBeforeRemove) {
                child1.beforeRemove();
            }
            child1.remove();
            this.child = child2;
            this.key = other.key;
        }
    }
    beforeRemove() {
        this.child.beforeRemove();
    }
    remove() {
        this.child.remove();
    }
    firstNode() {
        return this.child.firstNode();
    }
    toString() {
        return this.child.toString();
    }
}
export function toggler(key, child) {
    return new VToggler(key, child);
}
